﻿// ----------------------------------------------------------------------------------
// Sample Code from Fernando Machado Píriz's blog: http://fernandomachadopiriz.com
//
// <copyright company="Fernando Machado Píriz" file="DesktopIconsViewModel.cs">
// Copyright (c) Fernando Machado Píriz. Distributed under FSF's GPL v2.</copyright>
// ----------------------------------------------------------------------------------
// This code and information are provided "as is" without warranty of any kind,
// either expressed of implied, including but not limited to the implied warranties
// of merchantability and/or fitness for a particular purpose.
// ----------------------------------------------------------------------------------
namespace DesktopIcons
{
    using System;
    using System.Collections.Generic;
    using System.Configuration;
    using System.Diagnostics.CodeAnalysis;
    using System.Globalization;
    using System.Linq;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Input;
    using GivingAPresentation.Commons;
    using Microsoft.Win32;
    
    /// <summary>
    /// The view model of the <code>DesktopBackgroundView</code> using the Model View View Model pattern. An instance of this class is
    /// automatically created by the view as a window resource.
    /// </summary>
    public class DesktopIconsViewModel : BaseViewModel
    {
        /// <summary>
        /// A value indicating if desktop icons were disabled or not at the begining of the presentation.
        /// </summary>
        private bool changed;

        /// <summary>
        /// The backend field for <code>Model</code> property.
        /// </summary>
        private DesktopIconsModel model = new DesktopIconsModel();

        /// <summary>
        /// The backend field for <code>View</code> property.
        /// </summary>
        private DesktopIconsView view;

        /// <summary>
        /// Gets the model associated with this view model.
        /// </summary>
        public DesktopIconsModel Model
        {
            get
            {
                return this.model;
            }
        }

        /// <summary>
        /// Sets the view associated with this view model.
        /// </summary>
        /// <param name="value">The view to be associated with this view model.</param>
        public void AssignView(DesktopIconsView value)
        {
            this.view = value;
        }

        /// <summary>
        /// Closes or terminates the selected programs if the check box checkBoxCloseThesePrograms is checked.
        /// </summary>
        public void DoStartPresentation()
        {
            if (this.model.Presenting)
            {
                return;
            }

            this.model.Presenting = true;
            this.view.IsEnabled = false;

            // Hides desktop icons if they must be hidden; remembers is setting was changed to restore it later
            if (this.model.Hide && DesktopIconsHelper.Visible)
            {
                this.changed = true;
                DesktopIconsHelper.Visible = false;
            }
            else
            {
                this.changed = false;
            }
        }

        /// <summary>
        /// Starts the selected programs if the check box checkBoxCloseThesePrograms is checked.
        /// </summary>
        public void DoStopPresentation()
        {
            if (!this.model.Presenting)
            {
                return;   
            }

            // Shows desktop icons again if they were hidden
            if (this.model.Hide && this.changed)
            {
                DesktopIconsHelper.Visible = true;
            }

            this.model.Presenting = false;
            this.view.IsEnabled = true;
        }

        /// <summary>
        /// Loads model state.
        /// </summary>
        [SuppressMessage("Microsoft.Globalization", "CA1303:Do not pass literals as localized parameters",
            MessageId = "GivingAPresentation.Commons.ExceptionHelper.Check(System.Boolean,System.String)",
            Justification = "Resources will be used in a future version.")]
        public void DoInitializePart()
        {
            ExceptionHelper.Check(!this.model.Presenting, "Cannot load settings while giving a presentation.");

            try
            {
                this.model.Load();
            }
            catch (ConfigurationErrorsException e)
            {
                throw new GivingAPresentationException("Cannot load configuration.", e);
            }
        }

        /// <summary>
        /// Stores model state.
        /// </summary>                                                                   
        [SuppressMessage("Microsoft.Globalization", "CA1303:Do not pass literals as localized parameters",
            MessageId = "GivingAPresentation.Commons.ExceptionHelper.Check(System.Boolean,System.String)",
            Justification = "Resources will be used in a future version.")]
        public void DoFinalizePart()
        {
            ExceptionHelper.Check(!this.model.Presenting, "Cannot save settings while giving a presentation.");

            try
            {
                this.model.Save();
            }
            catch (ConfigurationErrorsException e)
            {
                throw new GivingAPresentationException("Cannot save configuration.", e);
            }
        }
    }
}
